#pragma rtGlobals=1		// Use modern global access method.
#pragma IgorVersion = 6.2
#pragma version = 4.4

// written by Matt Sfeir, Brookhaven National Laboratory
// msfeir@bnl.gov
// v.4.5 February 14. 2013


//////////////////////////////////////////////////////////////


Function TA_SolventResponse(w,x) : FitFunc
	Wave w
	Variable x

	//CurveFitDialog/ These comments were created by the Curve Fitting dialog. Altering them will
	//CurveFitDialog/ make the function less convenient to work with in the Curve Fitting dialog.
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ f(x) = A* Gauss(x,t0,sig)+B*hermiteGauss(1,((x-t0)/(2*sig^2)))*Gauss(x,t0,sig)+C*hermiteGauss(2,((x-t0)/(2*sig^2)))*Gauss(x,t0,sig)
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 1
	//CurveFitDialog/ x
	//CurveFitDialog/ Coefficients 5
	//CurveFitDialog/ w[0] = A
	//CurveFitDialog/ w[1] = B
	//CurveFitDialog/ w[2] = C
	//CurveFitDialog/ w[3] = t0
	//CurveFitDialog/ w[4] = sig

	return w[0]* Gauss(x,w[3],w[4])+w[1]*hermiteGauss(1,((x-w[3])/(2*w[4]^2)))*Gauss(x,w[3],w[4])+w[2]*hermiteGauss(2,((x-w[3])/(2*w[4]^2)))*Gauss(x,w[3],w[4])
End


//////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////


Function TA_1ExpGauss(w,t) : FitFunc
	Wave w
	Variable t

	//CurveFitDialog/ These comments were created by the Curve Fitting dialog. Altering them will
	//CurveFitDialog/ make the function less convenient to work with in the Curve Fitting dialog.
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ f(t) = y0+(amp1/G1)*ExpGauss(t-t0,G1,pw)
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 1
	//CurveFitDialog/ t
	//CurveFitDialog/ Coefficients 5
	//CurveFitDialog/ w[0] = y0
	//CurveFitDialog/ w[1] = pw
	//CurveFitDialog/ w[2] = t0
	//CurveFitDialog/ w[3] = amp1
	//CurveFitDialog/ w[4] = G1

	return w[0]+(w[3]/w[4])*ExpGauss(t-w[2],w[4],w[1])
End

//////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////

// Keep t0 and pulsewidth identical for all decays
// G are rates

Function TA_2ExpGauss(w,t) : FitFunc
	Wave w
	Variable t

	//CurveFitDialog/ These comments were created by the Curve Fitting dialog. Altering them will
	//CurveFitDialog/ make the function less convenient to work with in the Curve Fitting dialog.
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ f(t) = y0+(amp1/G1)*ExpGauss(t-t0,G1,pw)+(amp2/G2)*ExpGauss(t-t0,G2,pw)
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 1
	//CurveFitDialog/ t
	//CurveFitDialog/ Coefficients 7
	//CurveFitDialog/ w[0] = y0
	//CurveFitDialog/ w[1] = pw
	//CurveFitDialog/ w[2] = t0
	//CurveFitDialog/ w[3] = amp1
	//CurveFitDialog/ w[4] = G1
	//CurveFitDialog/ w[5] = amp2
	//CurveFitDialog/ w[6] = G2

	return w[0]+(w[3]/w[4])*ExpGauss(t-w[2],w[4],w[1])+(w[5]/w[6])*ExpGauss(t-w[2],w[6],w[1])
End

//////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////

// Keep t0 and pulsewidth identical for all decays
// G are rates

Function TA_3ExpGauss(w,t) : FitFunc
	Wave w
	Variable t

	//CurveFitDialog/ These comments were created by the Curve Fitting dialog. Altering them will
	//CurveFitDialog/ make the function less convenient to work with in the Curve Fitting dialog.
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ f(t) = y0+(amp1/G1)*ExpGauss(t-t0,G1,pw)+(amp2/G2)*ExpGauss(t-t0,G2,pw)+(amp3/G3)*ExpGauss(t-t0,G3,pw)
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 1
	//CurveFitDialog/ t
	//CurveFitDialog/ Coefficients 9
	//CurveFitDialog/ w[0] = y0
	//CurveFitDialog/ w[1] = pw
	//CurveFitDialog/ w[2] = t0
	//CurveFitDialog/ w[3] = amp1
	//CurveFitDialog/ w[4] = G1
	//CurveFitDialog/ w[5] = amp2
	//CurveFitDialog/ w[6] = G2
	//CurveFitDialog/ w[7] = amp3
	//CurveFitDialog/ w[8] = G3

	return w[0]+(w[3]/w[4])*ExpGauss(t-w[2],w[4],w[1])+(w[5]/w[6])*ExpGauss(t-w[2],w[6],w[1])+(w[7]/w[8])*ExpGauss(t-w[2],w[8],w[1])
End
//////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////

// Keep t0 and pulsewidth identical for all decays
// G are rates

Function TA_4ExpGauss(w,t) : FitFunc
	Wave w
	Variable t

	//CurveFitDialog/ These comments were created by the Curve Fitting dialog. Altering them will
	//CurveFitDialog/ make the function less convenient to work with in the Curve Fitting dialog.
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ f(t) = y0+(amp1/G1)*ExpGauss(t-t0,G1,pw)+(amp2/G2)*ExpGauss(t-t0,G2,pw)+(amp3/G3)*ExpGauss(t-t0,G3,pw)+(amp4/G4)*ExpGauss(t-t0,G4,pw)
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 1
	//CurveFitDialog/ t
	//CurveFitDialog/ Coefficients 11
	//CurveFitDialog/ w[0] = y0
	//CurveFitDialog/ w[1] = pw
	//CurveFitDialog/ w[2] = t0
	//CurveFitDialog/ w[3] = amp1	
	//CurveFitDialog/ w[4] = G1
	//CurveFitDialog/ w[5] = amp2
	//CurveFitDialog/ w[6] = G2
	//CurveFitDialog/ w[7] = amp3
	//CurveFitDialog/ w[8] = G3
	//CurveFitDialog/ w[9] = amp4
	//CurveFitDialog/ w[10] = G4
	

	return w[0]+(w[3]/w[4])*ExpGauss(t-w[2],w[4],w[1])+(w[5]/w[6])*ExpGauss(t-w[2],w[6],w[1])+(w[7]/w[8])*ExpGauss(t-w[2],w[8],w[1])+(w[9]/w[10])*ExpGauss(t-w[2],w[10],w[1])
End

//////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////

// Keep t0 and pulsewidth identical for all decays
// G are rates

Function TA_5ExpGauss(w,t) : FitFunc
	Wave w
	Variable t

	//CurveFitDialog/ These comments were created by the Curve Fitting dialog. Altering them will
	//CurveFitDialog/ make the function less convenient to work with in the Curve Fitting dialog.
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ f(t) = y0+(amp1/G1)*ExpGauss(t-t0,G1,pw)+(amp2/G2)*ExpGauss(t-t0,G2,pw)+(amp3/G3)*ExpGauss(t-t0,G3,pw)+(amp4/G4)*ExpGauss(t-t0,G4,pw)+(amp5/G5)*ExpGauss(t-t0,G5,pw)
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 1
	//CurveFitDialog/ t
	//CurveFitDialog/ Coefficients 13
	//CurveFitDialog/ w[0] = y0
	//CurveFitDialog/ w[1] = pw
	//CurveFitDialog/ w[2] = t0
	//CurveFitDialog/ w[3] = amp1	
	//CurveFitDialog/ w[4] = G1
	//CurveFitDialog/ w[5] = amp2
	//CurveFitDialog/ w[6] = G2
	//CurveFitDialog/ w[7] = amp3
	//CurveFitDialog/ w[8] = G3
	//CurveFitDialog/ w[9] = amp4
	//CurveFitDialog/ w[10] = G4
	//CurveFitDialog/ w[11] = amp5
	//CurveFitDialog/ w[12] = G5

	return w[0]+(w[3]/w[4])*ExpGauss(t-w[2],w[4],w[1])+(w[5]/w[6])*ExpGauss(t-w[2],w[6],w[1])+(w[7]/w[8])*ExpGauss(t-w[2],w[8],w[1])+(w[9]/w[10])*ExpGauss(t-w[2],w[10],w[1])+(w[11]/w[12])*ExpGauss(t-w[2],w[12],w[1])
End

//////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////


// Keep t0 and pulsewidth identical for all decays
// G are rates
// phi in degrees

Function TA_DampedOsc(w,t) : FitFunc
	Wave w
	Variable t

	//CurveFitDialog/ These comments were created by the Curve Fitting dialog. Altering them will
	//CurveFitDialog/ make the function less convenient to work with in the Curve Fitting dialog.
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ f(t) = y0+(amp1/G1)*ExpGauss(t-t0,G1,pw)*(cos(sqrt((1/Tper0)^2 +G1^2)*t+phi*(pi/180)))
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 1
	//CurveFitDialog/ t
	//CurveFitDialog/ Coefficients 7
	//CurveFitDialog/ w[0] = y0
	//CurveFitDialog/ w[1] = pw
	//CurveFitDialog/ w[2] = t0
	//CurveFitDialog/ w[3] = amp1
	//CurveFitDialog/ w[4] = G1
	//CurveFitDialog/ w[5] = Tper0
	//CurveFitDialog/ w[6] = phi

	return w[0] + (w[3]/w[4])*ExpGauss(t-w[2],w[4],w[1])*(cos(sqrt((1/w[5])^2 +w[4]^2)*t+w[6]*(pi/180)))
End

//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////

// Keep t0 and pulsewidth identical for all decays
// G are rates

Function TA_ExpDampedOsc(w,t) : FitFunc
	Wave w
	Variable t

	//CurveFitDialog/ These comments were created by the Curve Fitting dialog. Altering them will
	//CurveFitDialog/ make the function less convenient to work with in the Curve Fitting dialog.
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ f(t) = y0 + (amp1/G1)*ExpGauss(t-t0,G1,pw)*(cos(sqrt((1/Tper0)^2 +G1^2)*t+phi*(pi/180))) + (amp2/G2)*ExpGauss(t-t0,G2,pw)
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 1
	//CurveFitDialog/ t
	//CurveFitDialog/ Coefficients 9
	//CurveFitDialog/ w[0] = y0
	//CurveFitDialog/ w[1] = pw
	//CurveFitDialog/ w[2] = t0
	//CurveFitDialog/ w[3] = amp1
	//CurveFitDialog/ w[4] = G1
	//CurveFitDialog/ w[5] = Tper0
	//CurveFitDialog/ w[6] = phi
	//CurveFitDialog/ w[7] = amp2
	//CurveFitDialog/ w[8] = G2

	return w[0] + (w[3]/w[4])*ExpGauss(t-w[2],w[4],w[1])*(cos(sqrt((1/w[5])^2 +w[4]^2)*t+w[6]*(pi/180))) + (w[7]/w[8])*ExpGauss(t-w[2],w[8],w[1])
End

//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////

// Keep t0 and pulsewidth identical for all decays
// G are rates

Function TA_2ExpDampedOsc(w,t) : FitFunc
	Wave w
	Variable t

	//CurveFitDialog/ These comments were created by the Curve Fitting dialog. Altering them will
	//CurveFitDialog/ make the function less convenient to work with in the Curve Fitting dialog.
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ f(t) = y0 + (amp1/G1)*ExpGauss(t-t0,G1,pw)*(cos(sqrt((1/Tper0)^2 +G1^2)*t+phi*(pi/180))) + (amp2/G2)*ExpGauss(t-t0,G2,pw) + (amp3/G3)*ExpGauss(t-t0,G3,pw)
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 1
	//CurveFitDialog/ t
	//CurveFitDialog/ Coefficients 9
	//CurveFitDialog/ w[0] = y0
	//CurveFitDialog/ w[1] = pw
	//CurveFitDialog/ w[2] = t0
	//CurveFitDialog/ w[3] = amp1
	//CurveFitDialog/ w[4] = G1
	//CurveFitDialog/ w[5] = Tper0
	//CurveFitDialog/ w[6] = phi
	//CurveFitDialog/ w[7] = amp2
	//CurveFitDialog/ w[8] = G2
	//CurveFitDialog/ w[9] = amp3
	//CurveFitDialog/ w[10] = G3

	return w[0] + (w[3]/w[4])*ExpGauss(t-w[2],w[4],w[1])*(cos(sqrt((1/w[5])^2 +w[4]^2)*t+w[6]*(pi/180))) + (w[7]/w[8])*ExpGauss(t-w[2],w[8],w[1]) + (w[9]/w[10])*ExpGauss(t-w[2],w[10],w[1])
End

//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////



Function ExpPowerTail(w,t) : FitFunc
	Wave w
	Variable t

	//CurveFitDialog/ These comments were created by the Curve Fitting dialog. Altering them will
	//CurveFitDialog/ make the function less convenient to work with in the Curve Fitting dialog.
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ f(t) = y0+ A*exp(-t/tau)+B*t^(-alpha)
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 1
	//CurveFitDialog/ t
	//CurveFitDialog/ Coefficients 5
	//CurveFitDialog/ w[0] = A
	//CurveFitDialog/ w[1] = tau
	//CurveFitDialog/ w[2] = B
	//CurveFitDialog/ w[3] = alpha
	//CurveFitDialog/ w[4] = y0

	return w[4]+ w[0]*exp(-t/w[1])+w[2]*t^(-w[3])
End

//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////


Function Exp2PowerTail(w,t) : FitFunc
	Wave w
	Variable t

	//CurveFitDialog/ These comments were created by the Curve Fitting dialog. Altering them will
	//CurveFitDialog/ make the function less convenient to work with in the Curve Fitting dialog.
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ f(t) = y0+ A1*exp(-t/tau1)+A2*exp(-t/tau2)+B*t^(-alpha)
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 1
	//CurveFitDialog/ t
	//CurveFitDialog/ Coefficients 7
	//CurveFitDialog/ w[0] = A1
	//CurveFitDialog/ w[1] = tau1
	//CurveFitDialog/ w[2] = A2
	//CurveFitDialog/ w[3] = tau2
	//CurveFitDialog/ w[4] = B
	//CurveFitDialog/ w[5] = alpha
	//CurveFitDialog/ w[6] = y0

	return w[6]+ w[0]*exp(-t/w[1])+w[2]*exp(-t/w[3])+w[4]*t^(-w[5])
End

//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////


Function TA_ExpConvPower(pw, yw, xw) : FitFunc 
	WAVE pw, yw, xw
	// pw[0] = baseline offset
	// pw[1] = timezero
	// pw[2] = amplitude 
	// pw[3] = exponential rate constant
	// pw[4] = power order

	// Make a wave to contain an exponential with decay constant pw[3]. The 
	// wave needs enough points to allow any reasonable decay constant to 
	// get really close to zero. The scaling is made symmetric about zero to 
	// avoid an X offset from Convolve/A

	// Make power decay wave
	// set constant step size
	// 20 fs
	Variable dT = 0.02
	Variable resolutionFactor = 10
	// generate at least 10 lifetimes
	Variable nExpWavePnts = round(resolutionFactor/(pw[3]*dT))*2 + 1
	Make/D/O/N=(nExpWavePnts) expwave

	Variable nYPnts = max(numpnts(yw), nExpWavePnts)
	Make/D/O/N=(nYPnts) yWave

	// set scales
	setscale/P x -dT*(nExpWavePnts/2), dT, expwave
	setscale/P x xw[0],dT, yWave

	// fill in wave values
	expwave = ((x-pw[1])>=0)*exp(-pw[3]*(x-pw[1]))
	Variable sumexp 
	sumexp = sum(expwave, -inf,inf)
	expwave /= sumexp
	
	yWave =((x-pw[1])>0)*pw[2]*(((x-pw[1])^(-pw[4])))/(dT^(-pw[4])) 
	yWave = NumType(yWave) == 2 ? 0:yWave

	convolve/A expwave, yWave
	yWave = pw[0]+yWave

	yw = yWave(xw[p])

End

//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////


Function TA_GaussExp_ExpPower(pw, yw, xw) : FitFunc 
	WAVE pw, yw, xw
	// pw[0] = baseline offset
	// pw[1] = timezero
	// pw[2] = amplitude 
	// pw[3] = exponential rate constant
	// pw[4] = power order
	// pw[5] = pulsewidth
	// pw[6] = gaussexp ampl

	// Make a wave to contain an exponential with decay constant pw[3]. The 
	// wave needs enough points to allow any reasonable decay constant to 
	// get really close to zero. The scaling is made symmetric about zero to 
	// avoid an X offset from Convolve/A

	// Make power decay wave
	// set constant step size
	// 20 fs
	Variable dT = 0.02
	Variable resolutionFactor = 10
	// generate at least 10 lifetimes
	Variable nExpWavePnts = round(resolutionFactor/(pw[3]*dT))*2 + 1
	Make/D/O/N=(nExpWavePnts) expwave

	Variable nYPnts = max(numpnts(yw), nExpWavePnts)
	Make/D/O/N=(nYPnts) yWave

	// set scales
	setscale/P x -dT*(nExpWavePnts/2), dT, expwave
	setscale/P x xw[0],dT, yWave

	// fill in wave values
	expwave = ((x-pw[1])>=0)*exp(-pw[3]*(x-pw[1]))
	Variable sumexp 
	sumexp = sum(expwave, -inf,inf)
	expwave /= sumexp
	
	yWave =((x-pw[1])>0)*pw[2]*(((x-pw[1])^(-pw[4])))/(dT^(-pw[4])) 
	yWave = NumType(yWave) == 2 ? 0:yWave

	convolve/A expwave, yWave
	yWave = pw[0]+yWave + pw[6]*ExpGauss(x-pw[1],pw[3],pw[5])

	yw = yWave(xw[p])

End

//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////


Function TA_GaussExp2_ExpPower(pw, yw, xw) : FitFunc 
	WAVE pw, yw, xw
	// pw[0] = baseline offset
	// pw[1] = timezero
	// pw[2] = amplitude 
	// pw[3] = exponential rate constant
	// pw[4] = power order
	// pw[5] = pulsewidth
	// pw[6] = gaussexp ampl
	// pw[7] = gaussexp ampl 2
	// pw[8] = exponential rate constant 2
	
	// Make a wave to contain an exponential with decay constant pw[3]. The 
	// wave needs enough points to allow any reasonable decay constant to 
	// get really close to zero. The scaling is made symmetric about zero to 
	// avoid an X offset from Convolve/A

	// Make power decay wave
	// set constant step size
	// 20 fs
	Variable dT = 0.02
	Variable resolutionFactor = 10
	// generate at least 10 lifetimes
	Variable nExpWavePnts = round(resolutionFactor/(pw[3]*dT))*2 + 1
	Make/D/O/N=(nExpWavePnts) expwave

	Variable nYPnts = max(numpnts(yw), nExpWavePnts)
	Make/D/O/N=(nYPnts) yWave

	// set scales
	setscale/P x -dT*(nExpWavePnts/2), dT, expwave
	setscale/P x xw[0],dT, yWave

	// fill in wave values
	expwave = ((x-pw[1])>=0)*exp(-pw[3]*(x-pw[1]))
	Variable sumexp 
	sumexp = sum(expwave, -inf,inf)
	expwave /= sumexp
	
	yWave =((x-pw[1])>0)*pw[2]*(((x-pw[1])^(-pw[4])))/(dT^(-pw[4])) 
	yWave = NumType(yWave) == 2 ? 0:yWave

	convolve/A expwave, yWave
	yWave = pw[0]+yWave + pw[6]*ExpGauss(x-pw[1],pw[3],pw[5])+pw[7]*ExpGauss(x-pw[1],pw[8],pw[5])

	yw = yWave(xw[p])

End


//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////


Function TA_Gauss_ExpPower(pw, yw, xw) : FitFunc 
	WAVE pw, yw, xw
	// pw[0] = baseline offset
	// pw[1] = timezero
	// pw[2] = gausswidth
	// pw[3] = exp amplitude 
	// pw[4] = exponential rate constant
	// pw[5] = power ampl
	// pw[6] = power order
	
	Variable dt = 0.02
	Variable resolutionFactor = 20
	// generate at least 10 sigmas
	Variable nGWavePnts = round(((resolutionFactor*2.3*pw[2])/dT)*2) + 1
	Make/D/O/N=(nGWavePnts) gausswave
	
	Variable nYPnts = max(numpnts(yw), nGWavePnts)
	Make/D/O/N=(nYPnts) yWave
	
	// set scales
	setscale/P x -dT*(nGWavePnts/2), dT, gausswave
	setscale/P x xw[0],dT, yWave
	
	gausswave = Gauss(x,pw[1],pw[2])
	yWave = ((x-pw[1])>=0)*(pw[3]*exp(-(x-pw[1])*pw[4])+pw[5]*(x-pw[1])^(-pw[6]))
	yWave = NumType(yWave) == 2 ? 0:yWave
	duplicate/O ywave, ytest
	
	convolve/A gausswave, yWave
	yWave = pw[0]+yWave

	yw = yWave(xw[p])
	
End